/*************************************************************************\
*                                                                         *
*    File        : rx_sm.v                                                *
*    Description : Rx State Machine                                       *
*    Designer    : Mahmud Galela (mgalela@vlsi.itb.ac.id)                 *
*                                                                         *
*    The function of this module is to control all receive process.       *
*                                                                         *
\*************************************************************************/

`define INITIAL_COUNT   12'b000000000000
`define MAX_COUNT       12'b101111011100
`define STATUS_RESET    16'b0000000000000000
`define STATUS_ERROR    15'b000000000000000
`define eo_DA           12'b000000001011
`define GET_CRC_OUT     12'b000000001101
`define CRCO_ZERO        9'b000000000
`define Rx_IDLE          4'b000     
`define SFD_detect       4'b001  
`define Rx_RUN           4'b010      
`define Rx_HOLD          4'b011    
`define Rx_STOP          4'b100     
`define WRITE_STATUS     4'b101
`define HOLD1_STATUS     4'b110
`define HOLD2_STATUS     4'b111

 module rx_sm (  
  clk,          
  reset_n,      
  rx_dv,        
  rx_error,     
  SFD_detected, 
  address_match,
  multicast,    
  broadcast,    
  crc_ok,
  SFD_start_detect,
  data_en, 
  FCS_start_check,
  last_data,
  data_valid,
  rx_status_valid_n,
  rx_status
  );
	
  input		  clk,  
                  reset_n,
                  rx_dv, 
                  rx_error,
                  SFD_detected,
                  address_match,
                  multicast,
                  broadcast,
                  crc_ok; 
                
  output	  SFD_start_detect,
                  data_en,
                  FCS_start_check,
                  last_data,
                  data_valid,
                  rx_status_valid_n;
                
  output [15:0]   rx_status;
  
  reg		  SFD_start_detect,
                  FCS_start_check,
                  data_en,
                  last_data,
                  broadcast_temp,
                  multicast_temp,
                  address_match_temp,
                  data_valid,
                  rx_error_temp,
                  sending_data_n_status,
                  rx_status_valid_n;
                  
  reg [3:0]       rx_sm_state;
  reg [12:0]      nibble_count, 
                  nibble_count_temp;
  reg [15:0]      rx_status;
  
always @(posedge clk or negedge reset_n)
begin
  if (!reset_n) 
    rx_sm_state <= `Rx_IDLE;   
  else
    if (rx_error)
      begin
        rx_error_temp <= 1'b1;
        rx_sm_state <= `Rx_HOLD;   
      end
  else
     if((nibble_count_temp + 2'b10) == `MAX_COUNT)
       begin
         rx_error_temp <= 1'b1;
         rx_sm_state         <= `Rx_HOLD;    
       end
  else
  case (rx_sm_state)
  `Rx_IDLE :
       begin
         if(rx_dv)
         begin
           rx_sm_state           <= `SFD_detect;
           SFD_start_detect      <= 1'b1;           
         end
         else
	 begin
           SFD_start_detect      <= 1'b0;
           data_en               <= 1'b0;
           FCS_start_check       <= 1'b0;
           rx_status_valid_n     <= 1'b1;
           rx_status             <= `STATUS_RESET;
           last_data             <= 1'b0;
           data_valid            <= 1'b0;
           sending_data_n_status <= 1'b0;
           rx_sm_state           <= `Rx_IDLE;
           rx_error_temp         <= 1'b0;
         end
       end
  `SFD_detect :
       begin
         if(!rx_dv)
             rx_sm_state         <= `Rx_HOLD;
         else
         if (SFD_detected)
         begin
           SFD_start_detect      <= 1'b0;
           FCS_start_check       <= 1'b1;
           data_valid            <= 1'b1;
           sending_data_n_status <= 1'b1;
           data_en               <= 1'b1;          
           rx_sm_state           <= `Rx_RUN;
         end
         else
           rx_sm_state           <= `SFD_detect;
       end
  `Rx_RUN :
       begin
         if(!rx_dv)
         begin
             last_data           <= 1'b1;
             rx_sm_state         <= `Rx_HOLD;
         end
         else
            rx_sm_state           <= `Rx_RUN;
        end
  `Rx_HOLD :
       begin
         last_data               <= 1'b0;
         data_valid              <= 1'b0;
         data_en                 <= 1'b0;
         rx_sm_state             <= `Rx_STOP;
       end
  `Rx_STOP :
       begin
         FCS_start_check         <= 1'b0;                
         rx_sm_state             <= `WRITE_STATUS;
       end
  `WRITE_STATUS :
       begin
         rx_status_valid_n       <= 1'b0;
         rx_status[0]            <= rx_error_temp;
         rx_status[1]            <= address_match_temp;
         rx_status[2]            <= (broadcast_temp | multicast_temp);
         rx_status[3]            <= crc_ok;
         rx_status[15:4]   	 <= nibble_count + 1'b1;
         rx_sm_state 		 <= `HOLD1_STATUS;
       end
  `HOLD1_STATUS :
       begin
         rx_sm_state             <= `HOLD2_STATUS;                    
       end
  `HOLD2_STATUS :
         rx_sm_state             <= `Rx_IDLE;     
  default : 
       rx_sm_state               <= `Rx_IDLE;
  endcase
end

always @(posedge clk or negedge reset_n)
begin  
  if(!reset_n)
    nibble_count_temp <= `INITIAL_COUNT;
  else 
  if(!data_en)
    nibble_count_temp <= `INITIAL_COUNT;
   else 
    nibble_count_temp <= (nibble_count_temp + 1'b1);
end

always @(posedge clk or negedge reset_n)
begin
  if(!reset_n)
    nibble_count <= `INITIAL_COUNT;
  else
    if(data_en)
        nibble_count <= nibble_count_temp;
  else
     nibble_count <= nibble_count;
end

always @(posedge clk)
begin
  if(!reset_n)
    begin
      broadcast_temp     <= 1'b0;
      multicast_temp     <= 1'b0;
      address_match_temp <= 1'b0;
    end
  else
  if(!sending_data_n_status)
    begin
      broadcast_temp     <= 1'b0;
      multicast_temp     <= 1'b0;
      address_match_temp <= 1'b0;
    end
  else
  if (address_match)
    begin
      broadcast_temp <= broadcast;
      multicast_temp <= multicast;
      address_match_temp <= address_match;
    end
  else
    begin
      broadcast_temp     <= broadcast_temp;    
      multicast_temp     <= multicast_temp;    
      address_match_temp <= address_match_temp;
    end
end

endmodule 
